import appia.*;

import appia.protocols.udpsimple.*;
import appia.protocols.fifo.*;
import appia.protocols.remoteView.*;
import appia.protocols.stf.client.*;
import appia.protocols.stf.rrpc.*;

import appia.test.appl.*;

import java.net.InetAddress;


/*
 * Classe que define as pilhas de protocolos a usar
 * e inicializa-as.
 * O codigo da aplicacao deve ser feito
 * nas classes ClientLayer/ClientSession.
 */
public class ClientApplRRPC {

  public static void main(String args[]) {

    int port=-1;
    InetWithPort myAddress=null;
    InetWithPort gossip=null;

    //Trata os argumentos de entrada
    if(args.length!=2)
	invalidArgs();
      
    if(!args[0].equals("-gossip"))
	invalidArgs();
    
    try {
	int i=args[1].indexOf(":");
	gossip=new InetWithPort();
	gossip.host=InetAddress.getByName(args[1].substring(0,i));
	gossip.port=Integer.parseInt(args[1].substring(i+1));
    } catch(StringIndexOutOfBoundsException ex) {
	System.err.println("The host:port format wasn't respected in gossip");
	System.exit(1);
    } catch (NumberFormatException ex) {
	System.err.println("Invalid port in gossip");
	System.exit(1);
    } catch (java.net.UnknownHostException ex) {
	System.err.println("Invalid address in gossip");
	System.exit(1);
    }


    /* Cria as camadas*/
    Layer[] qos={
	new UdpSimpleLayer(),
	new FifoLayer(),
	new RemoteViewLayer(),
	new RRPCLayer(),
	new ClientLayer()
    };

    /* Cria o QoS */
    QoS myQoS=null;
    try {
	myQoS=new QoS("ClientRRPC QoS",qos);
    } catch(AppiaInvalidQoSException ex) {
	System.err.println("Invalid QoS");
	System.err.println(ex.getMessage());
	System.exit(1);
    }

    /* Cria o canal */
    Channel c=myQoS.createUnboundChannel("RRPC Channel");

    //Cria as sessoes que tem de ser inicializadas    
    ClientSession cs=(ClientSession)qos[qos.length-1].createSession();
    RemoteViewSession rvs=(RemoteViewSession)qos[qos.length-3].createSession();

    try{
	myAddress=new InetWithPort();
	myAddress.host=InetAddress.getLocalHost();
	myAddress.port=port;
    }catch(java.net.UnknownHostException e){
	System.err.println("Error in local address");
	System.exit(1);
    }

    //inicia as sessoes passado os parametros necesarios (morada propria e gossip)
    rvs.init(gossip);
    
    //cria cursor
    ChannelCursor cc=c.getCursor();
 
    //coloca as sessoes nas posicoes certas.
    try {
	cc.top();
	cc.setSession(cs);
	cc.down();
	cc.down();
	cc.setSession(rvs);
    } catch(AppiaCursorException ex) {
	System.err.println("Unexpected exception in main. Type code:"+
			   ex.type);
	System.exit(1);
    }                                                


    //inicia as outras sessoes
    try {
	c.start();
    } catch(AppiaDuplicatedSessionsException ex) {
	System.err.println("Sessions binding strangely resulted in "+
			   "one single sessions occurring more than "+
			   "once in a channel");
	System.exit(1);
    }

    //Poe o Appia a correr
    Appia.run();
  }

  private static void invalidArgs() {
    System.err.println("Invalid Parameters ");
    System.err.println("Usage: java ClientApplRRPC " +
		       "-gossip <GossipServer IP:port> ");


    System.exit(1);
  }

}
