import appia.*;

import appia.protocols.udpsimple.*;
import appia.protocols.fifo.*;
import appia.protocols.group.*;
import appia.protocols.group.bottom.*;
import appia.protocols.group.heal.*;
import appia.protocols.group.inter.*;
import appia.protocols.group.intra.*;
import appia.protocols.group.stable.*;
import appia.protocols.group.suspect.*;
import appia.protocols.group.sync.*;
import appia.protocols.remoteView.*;
//import appia.protocols.TotalSeq001.*;
import appia.protocols.stf.coordination.*;
import appia.protocols.stf.rrpc.*;
import appia.protocols.stf.server.*;

import appia.test.appl.*;

import java.net.InetAddress;


/*
 * Classe onde se define as varias camadas a usar
 * no lado do servidor e poe o Appia a correr 
 * Tudo o que tenha a ver com o codigo
 * do sevidor deve ser feito na camada
 * respectiva ("ServerLayer/ServerSession").
 * Aqui pode-se definir qual o algoritmo
 * de ordem total a utilizar.
 */
public class ServerApplRRPC {


  public static void main(String args[]) {

      int port=-1;
      InetWithPort gossip=null;

      /*Trata os argumentos*/
      if(args.length!=4)
	  invalidArgs();
      
      
      if(!args[0].equals("-port") || !args[2].equals("-gossip"))
	  invalidArgs();
    
      try{
	  port=Integer.parseInt(args[1]);
      } catch(NumberFormatException ex) {
	  System.err.println("Invalid port: "+args[1]);
	  System.exit(1);
      }    

      try {
	  int i=args[3].indexOf(":");
	  gossip=new InetWithPort();
	  gossip.host=InetAddress.getByName(args[3].substring(0,i));
	  gossip.port=Integer.parseInt(args[3].substring(i+1));
      } catch(StringIndexOutOfBoundsException ex) {
	  System.err.println("The host:port format wasn't respected in gossip");
                    System.exit(1);
      } catch (NumberFormatException ex) {
	  System.err.println("Invalid port in gossip");
	  System.exit(1);
      } catch (java.net.UnknownHostException ex) {
	  System.err.println("Invalid address in gossip");
	  System.exit(1);
      }
                                           





      /*define as layers para a comunicacao em grupo*/
      Layer[] stack = {
	  new UdpSimpleLayer(),
	  new FifoLayer(),
	  new GroupBottomLayer(),
	  new GossipOutExtLayer(),
	  new MergeOutLayer(),
	  new SuspectLayer(),
	  new SyncLayer(),
	  new IntraLayer(),
	  new InterLayer(),
	  new HealLayer(),
	  new RetransmitLayer(),
	  new StableLayer(),
//	  new TotalSeq001Layer(), //Protocolo de Ordem Total. Mudar se se quiser.
	  new CoordinationLayer(),
	  new ServerLayer()
      };


      
      
      /* Cria o QoS */
      QoS qoS=null;
      try {
	  qoS=new QoS("ServerRRPC QoS",stack);

      } catch(AppiaInvalidQoSException ex) {
	  System.err.println("Invalid QoS");
	  System.err.println(ex.getMessage());
	  System.exit(1);
      }

	  
      
      /* Cria o canal */
      Channel serverC=qoS.createUnboundChannel("RRPC Channel");
      
      //Cria explicitamente a sessao que vai ter de ser inicializada
      ServerSession ss=(ServerSession)stack[stack.length-1].createSession();      

      try {
	  //inicia a sessao passado os argumentos necessarios (porto e servidor gossip)
	  ss.init(port,gossip);
      } catch(java.net.UnknownHostException ex) {
	  System.err.println("At least one of the given addresses is invalid");
	  System.exit(1);
      } catch(AppiaGroupException e){
	  System.err.println("GroupException");
	  System.exit(1);
      }
      
      //cursor para percorrer a pilha
      ChannelCursor cc=serverC.getCursor();
 
      //posiciona as sessoes partilhadas
      try {
	  cc.top();
	  cc.setSession(ss);
      } catch(AppiaCursorException ex) {
	  System.err.println("Unexpected exception in main. Type code:"+
			     ex.type);
	  System.exit(1);
      }                         





      /* posiciona as outras camadas, por defeito */
      try {
	  serverC.start();
      } catch(AppiaDuplicatedSessionsException ex) {
	  System.err.println("Sessions binding strangely resulted in "+
			     "one single sessions occurring more than "+
			     "once in a channel");
	  System.exit(1);
      }
      
      // inicializa o Appia       
      Appia.run();
  }
    

    private static void invalidArgs() {
	System.err.println("Invalid Parameters ");
	System.err.println("Usage: java Appl " +
			   "-port <UDP port> "+
			   "-gossip <GossipServer IP:port> ");	
	System.exit(1);
  }

} 
