package appia.protocols.stf.coordination;

import appia.*;
import appia.events.*;
import appia.events.channel.*;
import appia.message.*;
import appia.protocols.udpsimple.RegisterSocketEvent;
import appia.protocols.udpsimple.InetWithPort;
import appia.protocols.group.*;
import appia.protocols.group.events.*;
import appia.protocols.group.intra.*;
import appia.protocols.group.sync.*;

import java.io.*;
import java.util.*;
import java.net.InetAddress;



/**
 * Classe que tem o codigo responsavel
 * pela parte generica do sistema RRPC
 * no lado do servidor.
 * E esta classe onde vao ter de fazer,
 * por exemplo, o codigo responsavel 
 * por enviar as respostas ao cliente.
 */
public class CoordinationSession extends Session {

    //canal de comunicacao
    private  Channel       channel=null; 

    /* Grupo */
    ViewState    vs;
    int          rank=-1;
    boolean      isBlocked=true;
    

    public CoordinationSession(CoordinationLayer l) {
	super(l);	
    }

    
    
    /**
     * Metodo chamado quando se recebe um evento.
     * Vai chamando os handlers respectivos.
     */
    public void handle(Event e) {	
	if(e instanceof ChannelInit)
	    handleChannelInit((ChannelInit)e);	
	else if(e instanceof View)
	    handleNewView((View)e);
	else if(e instanceof BlockOk)
	    handleBlock((BlockOk)e);	
	else if(e instanceof GroupSendableEvent)
	    handleGroupSendableEvent((GroupSendableEvent)e);
	else if(e instanceof SendableEvent)
	    handleSendableEvent((SendableEvent)e);
    }

    //trata uma nova vista
    //recuperacao a faltas deve ser feita aqui
    private void handleNewView(View e) {
	vs=e.vs;
	rank=e.ls.my_rank;
	isBlocked=false;
	
	try {
	    e.go();
	}
	catch(AppiaEventException ex) {
	    System.out.println("Exception while sending the new view event");
	}
    }

    //trata o BlockOk
    private void handleBlock(BlockOk e) {

	isBlocked=true;
	try {
	    e.go();
	}
	catch(AppiaEventException ex) {
	    System.out.println("Exception while forwarding the block ok event");
	}
    }

    //trata a inicializacao dos canais
    //esta deve ser a primeira mensagem enviada por cada camada.
    //inicializacoes devem ser feitas aqui.
    private void handleChannelInit(ChannelInit e) {

	try {
	    e.go();
	}
	catch(AppiaEventException ex) {
	    System.err.println("Unexpected exception in Server "+
			       "session");
	}

	channel=e.getChannel();

    }

    //trata uma mensagem do grupo
    //vai ser preciso adicionar o codigo
    private void handleGroupSendableEvent(GroupSendableEvent e){
    }

    //trata uma mensagem de cliente
    //vai ser preciso adicionar o codigo
    private void handleSendableEvent(SendableEvent e){	

    }

    /**
     * Retorna a Layer desta Session
     */
    public Layer getLayer() {
	return layer;
    }
}
