package appia.protocols.stf.server;

import appia.*;
import appia.events.*;
import appia.events.channel.*;
import appia.message.*;
import appia.protocols.udpsimple.RegisterSocketEvent;
import appia.protocols.udpsimple.InetWithPort;
import appia.protocols.group.*;
import appia.protocols.group.events.*;
import appia.protocols.group.sync.*;
import appia.protocols.group.intra.*;

import java.io.*;
import java.util.*;
import java.net.InetAddress;

/**
 * Classe onde vai estar o codigo do 
 * servidor.
 * Este codigo deve ser especifico da 
 * aplicacao.
 */
public class ServerSession extends Session {

    /* Appia */
    public  Channel       channel=null; 
    private int           myPort;

    /* Grupo */
    ViewState    vs;
    int          rank=-1;
    InetWithPort gossip[]=null;
    boolean      isBlocked=true;
    

    public ServerSession(ServerLayer l) {
	super(l);	
    }
    
    /**
     * Inicia a sessao.
     * @param myPort porto do servidor
     * @param gossip endereco do gossip
     */
    public void init(int myPort, InetWithPort gossip) throws AppiaGroupException, java.net.UnknownHostException {
	this.myPort=myPort;
	this.gossip=new InetWithPort[1];
	this.gossip[0]=gossip;
	InetWithPort myAddress=new InetWithPort();
	myAddress.host=InetAddress.getLocalHost();
	myAddress.port=myPort;
	makeView(myAddress);
	rank=0;
    }
    
    
    /**
     * metodo chamado quando se recebe um evento
     * @param e evento recebido.
     */    
    public void handle(Event e) {

	if(e instanceof ChannelInit)
	    handleChannelInit((ChannelInit)e);	
	else if(e instanceof View)
	    handleNewView((View)e);
	else if(e instanceof BlockOk)
	    handleBlock((BlockOk)e);	
	else if(e instanceof GroupSendableEvent)
	    handleGroupSendableEvent((GroupSendableEvent)e);
	else{
	    try{
		e.go();
	    }
	    catch(AppiaEventException ex){
		ex.printStackTrace();
		System.err.println("ERROR sending event");
	    }
	}
	
    }

    //trata a vista nova
    private void handleNewView(View e) {
	vs=e.vs;
	rank=e.ls.my_rank;
	isBlocked=false;

	System.out.println("New view delivered.");
	try {
	    e.go();
	}
	catch(AppiaEventException ex) {
	    System.out.println("Exception while sending the new view event");
	}
    }

    //trata o blockOk
    private void handleBlock(BlockOk e) {

	System.out.println("The grup was blocked. Impossible to send messages.");
	isBlocked=true;
	try {
	    e.go();
	}
	catch(AppiaEventException ex) {
	    System.out.println("Exception while forwarding the block ok event");
	}
    }


    //trata o ChannelInit
    private void handleChannelInit(ChannelInit e) {

	try {
	    e.go();
	}
	catch(AppiaEventException ex) {
	    System.err.println("Unexpected exception in Application "+
			       "session");
	}


	try {
	    RegisterSocketEvent rse=new
		RegisterSocketEvent(e.getChannel(),new Direction(Direction.DOWN),
				    this,myPort);
		rse.go();
	}
	catch(appia.AppiaEventException ex) {
	    switch(ex.type) {
	    case appia.AppiaEventException.UNWANTEDEVENT :
		System.err.println("The QoS definition doesn't satisfy the "+
				   "application session needs. "+
				   "RegisterSocketEvent, received by "+
				   "UdpSimpleSession is not being acepted");
		break;
	    default :
		System.err.println("Unexpected exception in "+
				   "Application session");
		break;
	    }
	}
    
       	channel=e.getChannel();
	

	try {
	    GroupInit gi=new GroupInit(vs,vs.view[rank],null,gossip,
				       channel,
				       new Direction(Direction.DOWN),this);
	    gi.go();
	}
	catch(AppiaEventException ex) {
	    ex.printStackTrace();
	    System.err.println("EventException while launching GroupInit");
	    
	}
	catch(NullPointerException ex) {
	    ex.printStackTrace();
	    System.err.println("EventException while launching GroupInit");
	}
	catch(AppiaGroupException ex) {
	    ex.printStackTrace();
	    System.err.println("EventException while launching GroupInit");
	}
	
	
	System.out.println("Open channel with name "+e.getChannel().getChannelID());

    }
    
    //Codigo quando se recebe uma mensagem do grupo.
    //Aqui deve se adicionar o codigo especifico destas mensagens.
    private void handleGroupSendableEvent(GroupSendableEvent e){
    }


    //cria a primeira vista.
    private void makeView(InetWithPort endereco)
	throws AppiaGroupException, java.net.UnknownHostException {

	Endpt vista[]= new Endpt[1];
	InetWithPort enderecos[]= new InetWithPort[1];

	enderecos[0]=endereco;

	vista[0]= new Endpt("ServerRRPC Endpt at"+ endereco.host.toString()+":"+endereco.port);

	Endpt coord=vista[0];
	

	vs=new ViewState( "versao1", new Group("groupID"),
			  new ViewID(0,coord),new ViewID[0],
			  vista,enderecos);
    }


    /**
     * Retorna um apontador para a Layer desta Session
     */
    public Layer getLayer() {
	return layer;
    }
}
